package com.logicbig.example;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

public class JRegex {
    public static void main(String[] args) {
        if (args.length == 1) {
            applyRegex(args[0], null, getInput());
        } else if (args.length == 2) {
            applyRegex(args[0], args[1], getInput());
        } else if (args.length == 3) {
            applyRegex(args[0], args[1], args[2]);
        } else {
            System.err.println("Wrong usage!");
            System.out.println("Valid uses:");
            System.out.println(" (1)  jregex \"pattern\" \"format\" \"text\"");
            System.out.println(" (2)  externalCommand | jregex \"pattern\"");
            System.out.println(" (3)  externalCommand | jregex \"pattern\" \"format\"");
            System.out.println(" (4)  jregex \"pattern\" < myfile.txt");
            System.out.println(" (5)  jregex \"pattern\" < myfile.txt");
        }
    }

    private static String getInput() {
        String input = "";
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(System.in))) {
            String line = null;
            while (true) {
                if ((line = reader.readLine()) != null) {
                    input += line + "\n";
                } else {
                    break;
                }
            }
        } catch (Exception e) {
            System.err.println(e);
        }
        return input.trim();
    }

    private static void applyRegex(String patternString, String groupFormat, String input) {
        try {
            if (groupFormat == null) {
                Pattern.compile(patternString)
                       .matcher(input)
                       .results()
                       .forEach(matchResult -> System.out.println(matchResult.group()));
            } else {
                Pattern pattern = Pattern.compile(patternString);
                Matcher m = pattern.matcher(input);
                while (m.find()) {
                    String formattedOutput = getFormattedOutput(m, input, groupFormat);
                    System.out.println(formattedOutput);
                }
            }
        } catch (PatternSyntaxException e) {
            System.err.println(e);
        }
    }

    private static String getFormattedOutput(final Matcher matcher,
                                             final String inputText,
                                             final String formattedReplacement) {
        StringBuilder result = new StringBuilder();
        int cursor = 0;
        while (cursor < formattedReplacement.length()) {
            char nextChar = formattedReplacement.charAt(cursor);
            if (nextChar == '\\') {
                cursor++;
                if (cursor == formattedReplacement.length())
                    throw new IllegalArgumentException("character to be escaped is missing");
                nextChar = formattedReplacement.charAt(cursor);
                result.append(nextChar);
                cursor++;
            } else if (nextChar == '$') {//does not support named groups
                // Skip past $
                cursor++;
                if (cursor == formattedReplacement.length())
                    throw new IllegalArgumentException(
                            "Illegal group reference: group index is missing");
                nextChar = formattedReplacement.charAt(cursor);
                int refNum = -1;
                // The first number is always a group
                refNum = nextChar - '0';
                if ((refNum < 0) || (refNum > 9))
                    throw new IllegalArgumentException(
                            "Illegal group reference");
                cursor++;
                // Capture the largest legal group string
                boolean done = false;
                while (!done) {
                    if (cursor >= formattedReplacement.length()) {
                        break;
                    }
                    int nextDigit = formattedReplacement.charAt(cursor) - '0';
                    if ((nextDigit < 0) || (nextDigit > 9)) { // not a number
                        break;
                    }
                    int newRefNum = (refNum * 10) + nextDigit;
                    if (matcher.groupCount() < newRefNum) {
                        done = true;
                    } else {
                        refNum = newRefNum;
                        cursor++;
                    }
                }
                // Append group
                if (matcher.start(refNum) != -1 && matcher.end(refNum) != -1)
                    result.append(inputText, matcher.start(refNum), matcher.end(refNum));
            } else {
                result.append(nextChar);
                cursor++;
            }
        }
        return result.toString();
    }
}